/* diversity-xmpp-backend - Communication with (diversity) XMPP server
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Daniel Willmann <daniel@openmoko.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <glib.h>
#include <diversity-world.h>
#include <diversity-bard.h>
#include "diversity-xmpp.h"
#include "diversity-xmpp-backend.h"
#include "xmpp-geoloc.h"

static struct {
	DiversityWorld *world;
	DiversityXmpp *xmpp;
} priv;

gboolean xmpp_update_position(DiversityXmpp *xmpp)
{
	static float x = 50, y = -12, z = 73, err = 1.34;

	xmpp_geoloc_publish_position(priv.xmpp, x, y, z, err);
	x += 0.045;
	y -= 0.002;
	err = err * 1.02;
	return TRUE;
}

int xmpp_init(DiversityWorld *world, GError **error)
{
	struct DiversityXmppUserInfo userinfo = {
		"daniel@thebe.totalueberwachung.de",
		"daniel",
		"",
		"diversity"
	};
	struct DiversityXmppServerInfo serverinfo = {
		"134.169.172.64",
		15222
	};

	if (!g_thread_supported()) {
		g_thread_init(NULL);
	}

	priv.world = world;
	g_object_ref(world);

	priv.xmpp = diversity_xmpp_new("xmpp", error);
	if (!priv.xmpp) {
		xmpp_fini();
		return FALSE;
	}

	diversity_xmpp_userinfo_set(priv.xmpp, &userinfo);
	diversity_xmpp_serverinfo_set(priv.xmpp, &serverinfo);
	diversity_xmpp_connect(priv.xmpp);

	/* XXX: Test function to update the position */
	g_timeout_add_seconds_full(G_PRIORITY_DEFAULT, 5, xmpp_update_position, priv.xmpp, NULL);

	return TRUE;
}

void xmpp_fini(void)
{
  GError *error = NULL;

	if (priv.xmpp) {
		g_object_unref(G_OBJECT(priv.xmpp));
		priv.xmpp = NULL;
	}

	if (priv.world) {
		g_object_unref(G_OBJECT(priv.world));
		priv.world = NULL;
	}

}
