/* diversity-tag.c - DiversityTag
 *
 * Copyright 2008 OpenMoko, Inc.
 * Authored by Chia-I Wu <olv@openmoko.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <time.h>
#include <glib.h>
#include <diversity-tag.h>
#include "diversity-marshal.h"

enum {
	MODIFIED,
	LAST_SIGNAL
};

enum {
	PROP_0,
};

#define DIVERSITY_TAG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), DIVERSITY_TYPE_TAG, DiversityTagPrivate))
typedef struct _DiversityTagPrivate DiversityTagPrivate;

struct _DiversityTagPrivate {
	time_t created;
	GHashTable *dict;
};

static guint tag_signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(DiversityTag, diversity_tag, DIVERSITY_TYPE_OBJECT);

static void diversity_tag_finalize(GObject *obj)
{
	DiversityTag *tag = DIVERSITY_TAG(obj);
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	g_hash_table_destroy(priv->dict);

	((GObjectClass *) diversity_tag_parent_class)->finalize(obj);
}

static void diversity_tag_class_init(DiversityTagClass *klass)
{
	GObjectClass *o_class = (GObjectClass *) klass;

	o_class->finalize = diversity_tag_finalize;

	g_type_class_add_private(klass, sizeof(DiversityTagPrivate));

	tag_signals[MODIFIED] =
		g_signal_new("modified",
			     G_TYPE_FROM_CLASS(klass),
			     G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
			     G_STRUCT_OFFSET(DiversityTagClass, modified),
			     NULL, NULL,
			     _diversity_marshal_VOID__STRING_STRING,
			     G_TYPE_NONE, 2,
			     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
			     G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
}

static void diversity_tag_init(DiversityTag *tag)
{
	DiversityObject *obj = DIVERSITY_OBJECT(tag);
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	obj->type = DIVERSITY_OBJECT_TYPE_TAG;

	priv->created = time(NULL);
	priv->dict = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
}

DiversityTag *diversity_tag_new(void)
{
	DiversityTag *tag;

	tag = g_object_new(DIVERSITY_TYPE_TAG, NULL);

	return tag;
}

void diversity_tag_set(DiversityTag *tag, const gchar *key, const gchar *val)
{
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	if (!key)
		return;

	if (val)
		g_hash_table_insert(priv->dict, g_strdup(key), g_strdup(val));
	else
		g_hash_table_remove(priv->dict, key);

	g_signal_emit(tag, tag_signals[MODIFIED], g_quark_from_string(key), key, val);
}

const gchar *diversity_tag_get(DiversityTag *tag, const gchar *key)
{
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	return g_hash_table_lookup(priv->dict, key);
}

void diversity_tag_foreach(DiversityTag *tag, GHFunc func, gpointer user_data)
{
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	g_hash_table_foreach(priv->dict, func, user_data);
}

const GHashTable *diversity_tag_get_hash_table(DiversityTag *tag)
{
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	return priv->dict;
}

GTime diversity_tag_get_created(DiversityTag *tag)
{
	DiversityTagPrivate *priv = DIVERSITY_TAG_GET_PRIVATE(tag);

	return (GTime) priv->created;
}
